/*
 *   COPYRIGHT NOTICE: This code was written by Markus Mummert. All
 *   rights reserved. FOR LICENCE TERMS, SEE ACCOMPANYING FILE "LICENCE".
 */
/*************************************************************************
 *
 *   Name    : fft.h
 *   Version : 1999/08/04 7.0
 *   Comment : Fast-Fourier-Transform (FFT) and inverse FFT
 *   Author  : Markus Mummert
 *   History : (see .c-file)
 *
 */
#ifndef FFT_INCLUDED
#define FFT_INCLUDED

#include "portab.h" /* allows FFTELEM to be defined in portab.h */
#ifndef FFTELEM     /* likely to be `double' */
#include "fftelem.h"
#endif

/* struct returned by fft_init() */
typedef struct {
	int     n,        /* read only */
	        ldn,      /* read only */
	        *irev;    /* internal use only */
	FFTELEM *r, *i,   /* <-- use r[], i[] to load/retrieve data */ 
	        *co, *si; /* internal use only */
} FFTDATA; 

extern FFTDATA *fft_init(/* int ldn */); /* prepare FFT of size n=2^ldn */
extern void fft(/* FFTDATA *datap */), ifft(/* FFTDATA *datap */);
extern void fft_discard(/* FFTDATA *datap */);

#endif /* FFT_INCLUDED */
/*
 * EOT
 *
 ************************************************************************/
