/*
 *   COPYRIGHT NOTICE: This code was written by Markus Mummert. All
 *   rights reserved. FOR LICENCE TERMS, SEE ACCOMPANYING FILE "LICENCE".
 */
/************************************************************************/
/*									*/
/*  Datei    :	KEY.H - Import/Export-Datei zu -> KEY.C			*/
/*  Version  :	1999/02/22 - 10						*/
/*  Autor    :	Markus Mummert						*/
/*									*/
/*  Kommentar:	Modul zum Management von Eingabeparametern bei komman-	*/
/*		dozeilenorientierten Programmen. Grundkonzept ist die	*/
/*		Zuordnung von Parametern aus Kommandozeile oder Datei	*/
/*		durch vorangestellte Schluesselwoerter (Keywords).	*/
/*									*/
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*									*/
/*  Folgenden Funktionen sind verfuegbar:				*/
/*									*/
/*	initkeytab()	Zuweisung von Eingabeparametern aus Kommando-	*/
/*			zeile, Dateien oder internen Defaultwerten	*/
/*			per Keywords an Programmvariablen. Zusatz-	*/
/*			funktionen. Benutzt Angaben aus Keytab (s.u.)	*/
/*	catmsg()	2 Strings verbinden, f. rekursive Fehlermeld.	*/
/*	errrorhandler()	Fehlermeldung ausgeben, Optik wie Initkeytab()	*/
/*	convspchar()	Wandlung von 'C'-Escape-Sequenzen in Strings	*/
/*	pt()		Makro - erleichtert Aufbau von Keytab		*/
/*									*/
/*  Typen und Konstanten:						*/
/*									*/
/*	KEY_TAB		Keytab-Typ (s.a.u.)				*/
/*	MAXLINE		max. Laenge der zu besetzenden STRING-Variablen	*/
/*  STRING,DOUBLE,INT	Formattypen in Keytab				*/
/*	EMPTY		Leerstring zum Ersatz von Kommentartext-,	*/
/* 			Default- oder Bereichseintraegen in Keytab	*/
/*									*/
/*  extern zu definieren:						*/
/*									*/
/*	header()	Aufruf von Hauptmodul-Infoausgabe (Hilfe)	*/
/*			sollte Programmkopf auf stderr ausgeben		*/
/*	"Keytab"	(Aufbau Keytab sie extra Kasten unten)		*/
/*									*/
/*  Synopsis zur Anwendung (weitere Infos in Source-Text):		*/
/*									*/
/*	#include "key.h"						*/
/*	#include <stdio.h>						*/
/*		(...)							*/
/*	char zB_string[MAXLINE];					*/
/*	double zB_d;							*/
/*	int zB_i;							*/
/*	KEY_TAB kt[] {							*/
/*		"<Keyword1>",pt(zB_d),DOUBLE, "<Text1>",		*/
/*			"<Default1>","<Untergrenze> <Obergrenze>",	*/
/*		"<Keyword2>",zB_string,STRING, "<Text2>",		*/
/*			"<Default2>","<Muster1> <Muster2> <..>",	*/
/*		"<Keyword3>",pt(zB_i),INT, "<Text3>",			*/
/*			"<Default3>","<Untergrenze> <Obergrenze>",	*/
/*		0L,0L,0L,0L,0L,0L					*/
/*	}								*/
/*		(...)							*/
/*	main(argc,argv)							*/
/*	int argc;							*/
/*	char *argv[];							*/
/*	{								*/
/*		(...)							*/
/*	    if (initkeytab(argc,argv,kt,"<Defaultparameterdatei>"))	*/
/*		exit(-1);						*/
/*		(...)							*/
/*	    if (Rechenfunktion_zB(zB_d,zB_string,zB_i,&errmsg)) {	*/
/*		catmsg("Rechenfunktion_zB",errmsg);			*/
/*		errorhandler(errmsg);					*/
/*		exit(-1);						*/
/*	    }								*/
/*	    exit(0);							*/
/*	}								*/
/*		(...)							*/
/*	header()							*/
/*	{								*/
/*		printf("\n    Rechenprogramm_zB, V1.0/9.9.99\n\n");	*/
/*	}								*/
/*		(...)							*/
/*									*/
/*	Wenn die Muster fuer STRING Leerzeichen enhalten sollen, 	*/
/*	kann man statt ' ' als Mustertrenner '*' verwenden. Somit	*/
/*	ist '*' ein verbotener Musterbestandteil. Moechte man		*/
/*	in Paramterdateien fuer STRING auch Leerzeichen verwenden,	*/
/*	so ist der Gesamtstring in '"' zu setzen. Auf der Kommando-	*/
/*	zeile gelten die Gruppierungsregeln der verwendeten Shell.	*/
/*	SAVEPARM, INFO und INFOPARM geben die Werte fuer STRING in	*/
/*	'"' aus, wenn Leerzeichen enthalten sind.			*/
/*                                                                      */
/************************************************************************/

#ifndef KEY_INCLUDED			/* Schutz vor mehrfach-include	*/
#define KEY_INCLUDED

/*----------------------------------------------------------------------*/
/*  Import - wird zu Export:						*/
/*----------------------------------------------------------------------*/

extern header();			/* Textbeitrag zu Helpfunktion	*/

/*----------------------------------------------------------------------*/
/*  Import - exclusiv fuer Source:					*/
/*----------------------------------------------------------------------*/

#ifdef KEY_SOURCE
/* siehe key.c */
#endif	

/*----------------------------------------------------------------------*/
/*  Export :								*/
/*----------------------------------------------------------------------*/

extern int initkeytab();		/* Parameter-Management		*/
extern char *catmsg();			/* Zugabe: Strings verbinden	*/
extern errorhandler();			/* Zugabe: Fehlertextausgabe	*/
extern char *convspchar();		/* Zugabe: Escape-Seq. Konvert.	*/

#define MAXLINE	256			/* max. Laenge f. alle Strings	*/
					/* auch catmsg(), incl.	'\0'	*/

#define pt(var)	((char*)&(var))		/* Makro fuer DOUBLE- und INT-	*/
					/* Format in Keytab		*/
#define STRING	"%s"			/* Variablenformate Keytab...	*/
#define DOUBLE	"%lf"
#define INT	"%d"
#define EMPTY	""			/* fuer Auslassung Wertebereich	*/

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*  KEY_TAB-Typ definiert pro Zeile 6 vorzubesetzende Textzeiger:	*/
/*									*/
/*		1. Keyword (wg. Fehlertext in Grossbuchst.)		*/
/*		2. (char*)Zeiger zu besetzender Variable (Makro pt())	*/
/*		3. Variablenformat (Konstante STRING, DOUBLE oder INT)	*/
/*		4. ein den Parameter kommentierender Text (< 38 Zchn.)	*/
/*		5. Default-Variablenwert als String, Konstante EMPTY	*/
/*		   erlaubt es, auf Default-Angabe zu verzichten		*/
/*		6. Wertebereich fuer Variablenformat DOUBLE und INT:	*/
/*			String mit zwei durch Leerzeichen getrennten	*/
/*			Zahlen als Unter- bzw. Obergrenze		*/
/*		   oder Zulassungsmustersammlung fuer STRING:		*/
/*			String mit durch Leerzeichen getrennten	Teil-	*/
/*			Strings, die die zulaessigen Eingaben 		*/
/*			repraesentieren.				*/
/*		   Angabe der Konstante EMPTY erlaubt Parameterangaben	*/
/*		   ohne Beschraenkung.					*/
/*									*/
/*		Der Abschluss der Tabelle erfolgt durch eine NULL-	*/
/*		Zeigerzeile.						*/
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct {
	char *kwd, *kvar, *kfmt, *ktxt, *kval, *krge;
} KEY_TAB;


#endif					/* KEY_INCLUDE			*/
/*******Dateiende********************************************************/
