/*
 *   COPYRIGHT NOTICE: This code was written by Markus Mummert. All
 *   rights reserved. FOR LICENCE TERMS, SEE ACCOMPANYING FILE "LICENCE".
 */
/************************************************************************/
/*									*/
/*  Datei    :	OLA.C (main)						*/
/*  Version  :	(see below)						*/
/*  Autor    :	Markus Mummert						*/
/*									*/
/*  Kommentar:	Reconstruct signal from its sliding discrete		*/
/*		Fourier-transform using inverse DFT and overlap-add	*/
/*									*/
/*  history  :	1999/09/25 begin based on DFT.C/V1.2/1999/08/25		*/
/*		(continued below)					*/
/*									*/
/************************************************************************/
/*
 *  $Source: /home/mummert/src/MUMTOOLS/ola/RCS/ola.c,v $
 *  $Revision: 1.0 $
 *  $Date: 1999/11/14 23:06:00 $
 *  $Author: mummert $
 *  $Log: ola.c,v $
 *  Revision 1.0  1999/11/14 23:06:00  mummert
 *  First release
 *
 */
	
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "portab.h"			/* portability			*/
#include "key.h"			/* keytab-management		*/
#include "window.h"
#include "winhelp.h"
#include "fft.h"
#include "numio.h"

int	DEBUG = 0;

#define	DFLTPARMFILE	"ola.par"	/* default parameter file	*/

/*----------------------------------------------------------------------*/
/*	global variables						*/
/*----------------------------------------------------------------------*/

char 	g_ifile[MAXLINE],		/* input file name		*/
	g_ifields[MAXLINE],		/* input fields			*/
	g_ofile[MAXLINE],		/* output file name		*/
	g_wfile[MAXLINE],		/* window file name		*/
	g_ifmt[MAXLINE],		/* input format			*/
	g_ofmt[MAXLINE],		/* output format		*/
	g_wfmt[MAXLINE],		/* window format		*/
	g_ofldfmt[MAXLINE],		/* output-printf()-format	*/
	g_ofsep[MAXLINE],		/* output field spearator	*/
	g_ofields[MAXLINE],		/* output fields		*/
	g_wtype[MAXLINE],		/* window type			*/
	g_wsmpl[MAXLINE],		/* window sampling		*/
	g_transform[MAXLINE],		/* transform type		*/
	g_sn[MAXLINE],			/* block size N (string)	*/
	g_fixa[MAXLINE],		/* fix level of analytic input	*/
	g_debug[MAXLINE],		/* debug-switch			*/
	g_splfile[MAXLINE],		/* ftt-format level file name	*/
	g_sppfile[MAXLINE];		/* ftp-format phase file name	*/

double  g_scale_usr,			/* user-gain			*/
	g_scale,			/* internal gain		*/
	g_f3db,				/* 3dB-cutoff-f some windows.	*/
	g_fco,				/* cutoff-f for sinc window	*/
	g_walpha,			/* trade-off-parm. (Harris78)	*/
	g_wraise,			/* window-raise			*/
	g_fa;				/* sampling frequency		*/

int	g_n,				/* block size N			*/
	g_m,				/* block advance M		*/
	g_ifibeg,			/* input from f-idx (modulo n)	*/
	g_ifihop,			/* input idx distance (mod n)	*/
	g_ifiend,			/* input stop f-idx (modulo n)	*/
	g_tshift,			/* time-shift for causal-IIR	*/
	g_nwdow,			/* window size L		*/
	g_nofields;			/* number of fields in output	*/

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
					/* Keytab (vergl. key.h/c)	*/
KEY_TAB kt[]= {
"IFILE",	g_ifile,	STRING, 
				"Input file (stdin: '-')",
				"-",	EMPTY,
"IFMT",		g_ifmt,		STRING, 
				"IFILE: file format",
				"ascii",	
"b u w l d f a byte ubyte word long double float ascii",
"IFIELDS",	g_ifields,	STRING, 
				"IFILE: fields (ri:real/img, lp:lev/phase)",
				"lp",	"ri lp",
"SPLFILE",	g_splfile,	STRING,
				"ftt-level file (no IFILE, needs SPPFILE)",
				"-",	EMPTY,
"SPPFILE",	g_sppfile,	STRING,
				"ftp-phase file (no IFILE, needs SPLFILE)",
				"-",	EMPTY,
"TRANSFORM",	g_transform,	STRING,
				"Transform type (dft, idft, none)",
				"idft",	"dft idft none",
"N",		g_sn,		STRING,	
				"Transform block-size (powers of 2 only)",
				"1024",	
"2 4 8 16 32 64 128 256 512 1024 2048 4096 8192 16384 32768 65536\
 131072 262144 524288 1048576",
"IBEGIDX",	pt(g_ifibeg),	INT,
				"f-idx to begin assigning input lines with",
				"0",	"0 2000000000",
"IHOPIDX",	pt(g_ifihop),	INT,
				"f-indices advance after each assignment",
				"1",	"0 2000000000",
"IENDIDX",	pt(g_ifiend),	INT,
				"f-idx of last assig. (-1: =N-1, -2: =N/2)",
				"-2",	"-2 2000000000",
"FIXANALYTIC",	g_fixa,		STRING,
				"Multiply spectrum by 2 except at i=0,N/2",
				"auto",	"auto yes no a y n ",
"FSAMP", 	pt(g_fa),	DOUBLE,	
				"Signal sample-f (needed for WF3DB, WFCUT)",
				"1.0",
				"1E-50 1E+50",	
"L",		pt(g_nwdow),	INT,	
				"Window length (0: =N); zero-pad N-L",
				"0",	"0 1048576",	
"WINDOW",	g_wtype,	STRING,	
				"Window type (see 'WINDOW help')",
				"rectangle",
				EMPTY,
"WSAMPLING",	g_wsmpl,	STRING,
				"auto: (N>L:symmetric, N=L:periodic)",
				"auto",	"auto symmetric periodic a s p",
"WALPHA",	pt(g_walpha),	DOUBLE,	
				"Window parameter as defined by Harris78",
				"1.0",
				EMPTY,	
"WFCUT",	pt(g_fco),	DOUBLE,	
				"Cutoff-frequency for window 'sinc'",
				"0.25",	EMPTY,
"WF3DB",	pt(g_f3db),	DOUBLE,	
				"f3dB for 'gaussian-f3db'/causal-IIR-wdows",
				"0.002",
				EMPTY,
"WSHIFT",	pt(g_tshift),	INT,	
				"Sample-shift t=0 of causal-IIR-windows",
				"0",	EMPTY,
"WRAISE",	pt(g_wraise),	DOUBLE,	
				"Raise wdow by WRAISE+(1-WRAISE)*w[i]/wmax",
				"0.0",	EMPTY,
"EXTWFILE",	g_wfile,	STRING, 
				"External window file >= L ('-': internal)",
				"-",	EMPTY,
"EXTWFMT",	g_wfmt,		STRING, 
				"External window file-format",
				"ascii",	
"b u w l d f a byte ubyte word long double float ascii",
"M",		pt(g_m),	INT, 
				"Block advance (0: =N)",
				"0",	"0 1048576",
"SCALE",	pt(g_scale_usr),DOUBLE,	
				"Gain (also depends on A&S window-product)",
				"1.0", 
				EMPTY,	
"OFILE",	g_ofile,	STRING,
				"Output file (stdout: '-' pipe: '|...')",
				"-",
				EMPTY,
"OFMT",		g_ofmt,		STRING,	
				"OFILE: file format",
				"word",	
"b u w l d f a byte ubyte word long double float ascii",
"OFIELDS",	g_ofields,	STRING,	
				"OFILE: data fields (r, i, ri, ir)",
				"r",
				EMPTY,
"OFIELDFMT",	g_ofldfmt,	STRING,	
				"OFMT=ascii: printf()-format data fields",
				"%+le",
				EMPTY,
"OFIELDSEP",	g_ofsep,	STRING,	
				"OFMT=ascii: data field separator",
				"\\t",
				EMPTY,
/*
 * "DEBUG",	g_debug,	STRING,	
 * 				"Debug output",
 *				"no",	"y n yes no",
 */
0L,0L,0L,0L,0L,0L };

/*----------------------------------------------------------------------*/
/*  Write header of usage message, to be called by key.c		*/
/*----------------------------------------------------------------------*/
header()
{
char string[90], version[90];

sscanf("$Revision: 1.0 $ $Date: 1999/11/14 23:06:00 $ $Author: mummert $",
        "$%*s %s $ $%*s %s %*s $ $%*s %s $",
        string, string+30, string+60);
sprintf(version, "OLA V%s %s %s", string, string+30, string+60);
fprintf(stderr, "\n");
fprintf(stderr, "    %s\n\n", version);
fprintf(stderr, 
"    Reconstruct signal from its sliding discrete Fourier-transform\n");
fprintf(stderr, 
"    by performing overlap-add of windowed inverse transform of each block\n");
fprintf(stderr, "\n");
}
/*----------------------------------------------------------------------*/
/*  Set up <in> to contain next block by reading/convert data		*/
/*  Read next spectrum block from input (IFILE or SPLFILE/SPPFILE)	*/
/*  Upon return <in> contains <nin> numbers, alternating real and	*/
/*  imaginary part in sequence. Numbers actually read are returned.	*/
/*  Allocated size of <in> has to be 2 times max(nin).			*/
/*----------------------------------------------------------------------*/
int read_next(istream, splstream, sppstream, in, nin)
FILE *istream, *splstream, *sppstream;
double *in;
int nin;
{
	int i, read;
	double amp, ph, x;

	if (istream != NULL) {			/* input from IFILE	*/
	    read = numread(istream, g_ifmt, in, nin);
	    if ((g_ifields[0] | 0x20) == 'l') {	/* convert lev/ph ...	*/
		for (i = 0; i < nin/2; i++) {
		    amp = exp(in[2*i] * M_LN10/20.0);
		    ph = in[2*i+1];
		    in[2*i]   = amp * cos(ph);	/* ... to real/img	*/
		    in[2*i+1] = amp * sin(ph);
		}
	    }
	} else {				/* input is SP[LP]FILE	*/
		/*
		 * UGLY: Array size of in[] must be 2*nin: 
		 * Load level in 3rd and phase in 4th quarter of
		 * allocated array; then convert level and phase into
		 * alternating real/img sequence in first half of array
		 */
		read =  numread(splstream, "u", in+nin, nin/2);
		read += numread(sppstream, "u", in+3*nin/2, nin/2);
		for (i = 0; i < nin/2; i++) {
		    if ((x = in[nin+i]) > 0) {
			amp = exp((256.0 * x - 180.0)/2.0 * M_LN10/20.0);
		    } else {
			amp = 0;
		    }
		    ph = in[3*nin/2+i] * (2*M_PI);
		    in[2*i]   = amp * cos(ph);	/* real */
		    in[2*i+1] = amp * sin(ph);	/* img  */
		}
	}	
	return(read);
}
/*----------------------------------------------------------------------*/
/*  Write <count> numbers from <out>, starting at <head> to <ostream>,	*/
/*  wrapping back to begin of <out> if size is surpassed		*/
/*----------------------------------------------------------------------*/
output(out, head, count, ostream)
double *out;
int head, count;
FILE *ostream;
{
	int size, check;
	char *errmsg;

	size  = g_n * g_nofields;
	if (head + count > size) {
		/*
		 * output wraps over end of array 'out', have to write
		 * two portions; recursive call will handle first portion;
		 * this avoids examining return-codes of two numwrite() calls
		 */
		output(out, head, size - head, ostream);
		count -= (size - head);
		head = 0;
	}
	/*
	 * reamining output can be written in one portion
	 */
	check = numwrite(ostream, g_ofmt, out + head, count, 
				 g_nofields, g_ofldfmt, g_ofsep);
	if (check != count)  {
		if (check == -count) {
			errorhandler("overflow when converting to OFMT" );
			exit(-1);
		}
		errmsg = catmsg(g_ofile, "when writing");
		errorhandler(errmsg);
		exit(-1);
	}
}
/*----------------------------------------------------------------------*/
/*  Initialize overlap-add array <out> and associated pointer <*headp>	*/
/*----------------------------------------------------------------------*/
ola_init(out, headp, ostream)
double *out;
int *headp;
FILE *ostream;
{
	int i;

	*headp = 0;
	for (i = 0; i < g_nofields * g_n; i++)
		out[i] = 0.0;
}
/*----------------------------------------------------------------------*/
/*  Overlap-add <xr>, <xi> into <out> array starting at <*headp> and	*/
/*  applying synthesis window <wdow>, then output overlap-portion to	*/
/*  ostream. The overlap-portion in <out> is finally set to zero; this	*/
/*  allows the OLA-procedure to take place in a revolving buffer.	*/
/*----------------------------------------------------------------------*/
ola_next(xr, xi, wdow, out, headp, ostream)
double *xr, *xi, *wdow, *out;
int *headp;
FILE *ostream;
{
	int i, k, l, size, ovlp;
	char *errmsg, s[MAXLINE];

	size = g_n * g_nofields;		/* size of out[]	*/
	ovlp = g_m * g_nofields;		/* overlap in out[]	*/

	l = *headp;				/* window and OLA	*/
	for (i = 0; i < g_nwdow; i++) {
	    for (k = 0; k < g_nofields; k++) {
		switch (g_ofields[k] | 0x20) {
		case 'r': out[l] += (xr[i] * wdow[i]); break;
		case 'i': out[l] += (xi[i] * wdow[i]); break;
		default:
		    sprintf(s, "unknown field specifier `%c' in OFIELDS",
			    g_ofields[k]);
		    errorhandler(s);
		    exit(-1);
		}
		l = (l+1) % size;
	    }
	}
						/* output overlap	*/ 
	output(out, *headp, ovlp, ostream);

	l = *headp;				/* zero overlap		*/
	for (i = 0; i < ovlp; i++) {
		out[l] = 0.0;
		l = (l+1) % size;
	}

	*headp = (*headp + ovlp) % size;
}
/*----------------------------------------------------------------------*/
/*  Flush remainder in <out> from final ola_next() call to <ostream>	*/
/*----------------------------------------------------------------------*/
ola_flush(out, headp, ostream)
double *out;
int *headp;
FILE *ostream;
{
	int size, count;

	size  = g_n * g_nofields;
	count = (g_nwdow - g_m) * g_nofields;
	output(out, *headp, count, ostream);
	*headp = (*headp + count) % size;
}
/*----------------------------------------------------------------------*/
/*  load window								*/
/*----------------------------------------------------------------------*/
void load_window(wstream, wdow)
FILE *wstream;
double *wdow;
{
	int i;
	char *errmsg;
	double wnorm;

	if (numread(wstream, g_wfmt, wdow, g_nwdow) != g_nwdow) {
		errmsg = catmsg(g_wfile, "not enough (<L) samples found");
		errorhandler(errmsg);
		exit(-1);
	}
	wnorm = 0.0;			/* normalize			*/
	for (i = 0; i < g_nwdow; i++)
		wnorm += wdow[i];
	if (fabs (wnorm) < 1E-30) {
		errmsg = catmsg(g_wfile, "has zero gain (arithmetic mean)");
		errorhandler(errmsg);
		exit(-1);
	}
	wnorm /= g_scale;
	for (i = 0; i < g_nwdow; i++)
		wdow[i] /= wnorm;	    
}
/*----------------------------------------------------------------------*/
/*  Make window								*/
/*----------------------------------------------------------------------*/
void make_window(wdow)
double *wdow;
{
	int i, wsampling;
	char *p, w[MAXLINE], em[MAXLINE];


	wsampling = WINDOW_SYMMETRIC; 
	switch (g_wsmpl[0] | 0x20) {
	case 'a': if (g_nwdow == g_n)
			wsampling = WINDOW_PERIODIC;
		  break;
	case 'p': wsampling = WINDOW_PERIODIC; 
		  break;
	case 's': break;
	}

	p = g_wtype; i = 0;
	sscanf(p += i, "%[0-9a-zA-Z-]%n", w, &i);
	do {
	    if(!strcmp(w, "list")) {
		list_of_windows(stdout);
		exit(-1);
	    } else if(!strcmp(w, "help")) {
		help_on_windows(stdout);
		exit(-1);
	    } else if(!strcmp(w, "notes")) {
		notes_on_windows(stdout);
		exit(-1);
	    }
	    switch (window_window(wdow, g_nwdow, w, g_scale, wsampling,
		    g_walpha, g_f3db/g_fa, g_fco/g_fa, g_tshift)) {
	    case WINDOW_OK:
		break;
	    case WINDOW_ERROR_NOMEM:
		errorhandler("not enough memory");
		exit(-1);
	    case WINDOW_ERROR_ZERODC:
		errorhandler("resulting window is zero due to TSHIFT");
		exit(-1);
	    case WINDOW_ERROR_PRANGE:
		sprintf(em, 
		    "WINDOW parameter out of range for `%s'", w);
		errorhandler(em);
		exit(-1);
	    case WINDOW_ERROR_NUMERIC:
		sprintf(em, 
		    "numerical error calculating window `%s'", w);
		errorhandler(em);
		exit(-1);
	    case WINDOW_ERROR_NOWIN:
		sprintf(em, 
		    "unknown type `%s' in WINDOW, try `WINDOW help'", w);
		errorhandler(em);
		exit(-1);
	    default:
		errorhandler("UNEXPECTED: window_window(): unknown error");
		exit(-1);
	    }
	} while (sscanf(p += i, "%[*]%[0-9a-zA-Z-]%n", w, w, &i) >= 2);

	if (fabs(g_wraise) < 1E-30)
	        return;
	switch (window_raise(wdow, g_nwdow, g_wraise, g_scale)) {
	case WINDOW_OK:
		break;
	case WINDOW_ERROR_ZERODC:
		errorhandler("resulting window has zero gain due to RAISE");
		exit(-1);
	default:
		errorhandler("UNEXPECTED: window_raise(): unknown error");
		exit(-1);
	}
}	
/*----------------------------------------------------------------------*/
/*  Central processing loop						*/
/*----------------------------------------------------------------------*/
do_calc(in, nin, fixflag, istream, wstream, wdow, fftp, out, ostream, 
	splstream, sppstream)
double *in, *wdow, *out;
int nin, fixflag;
FFTDATA *fftp;
FILE *istream, *ostream, *wstream, *splstream, *sppstream;
{
	int i, k, head, check, n2;
	char *errmsg;
	FFTELEM *xr, *xi;

	if (wstream != NULL)
	    load_window(wstream, wdow);	/* load	external window		*/
	else
	    make_window(wdow);		/* calculate internal window	*/

	n2 = g_n/2;
	xr = fftp->r;
	xi = fftp->i;
	if (read_next(istream, splstream, sppstream, in, nin) != nin) {
		errmsg = catmsg(g_ifile, "out of input data in first block");
		errorhandler(errmsg);
		exit(-1);
	};
	ola_init(out, &head, ostream);		/* init overlapp-add 	*/

	while(-1) {
	    for (i = 0; i < g_n; i++) {		/* zero-set block	*/
		xr[i] = 0;
 		xi[i] = 0;
	    }
	    k = 0;
	    i = g_ifibeg;
	    /* IMPORTANT: expecting to be end==beg+(nin/2-1)*hop */ 
	    while (-1) {			/* load data in block	*/
		xr[i] = (FFTELEM)(in[k++]);
		xi[i] = (FFTELEM)(in[k++]);
		if (i == g_ifiend)
			break;
		i = (i+g_ifihop+g_n)%g_n;
	    }

	    if (fixflag) {			/* fix analytic	spectr.	*/
		xr[0] /= 2;
		xi[0] /= 2;
		xr[g_n/2] /= 2;
		xi[g_n/2] /= 2;
	    }

	    if (g_transform[0] == 'i')
		ifft(fftp);			/* inverse FFT, or	*/
	    else if (g_transform[0] == 'd')
		fft(fftp);			/* FFT, or nothing	*/

						/* windowing,OLA,output	*/
	    ola_next(xr, xi, wdow, out, &head, ostream);

	    check = read_next(istream, splstream, sppstream, in, nin);
	    if (check != nin) {			/* EOF, flush OLA	*/	
		ola_flush(out, &head, ostream);
		if (check == 0)			/* normal EOF		*/	
		    return;
		errmsg = catmsg(g_ifile, "out of input data in last block");
		errorhandler(errmsg);
		exit(-1);
	    };
	}
}
/*----------------------------------------------------------------------*/
/*  main								*/
/*----------------------------------------------------------------------*/
main(argc, argv)
int argc;
char *argv[];
{
	char *errmsg, c;
	double *in, *out, *wdow;
	FILE *istream, *ostream, *wstream, *splstream, *sppstream;
	FFTDATA *fftp;
	int d, k, i, fftsize, ldn, check, nin, fixflag;

					/* keytab-mechanism		*/
	if (initkeytab(argc, argv, kt, DFLTPARMFILE))
		exit(-1);
	if ((g_debug[0] | 0x20) != 'n')
		DEBUG = 1;

					/* post-process keytab values..	*/
	sscanf(g_sn, "%ld", &g_n);
	ldn = -1;
	while ((g_n >> 1) >> ++ldn)
	  ;
	fftsize = 1 << ldn; 
	if (fftsize != g_n) {
 		errorhandler("UNEXPECTED: fftsize != g_n");
		exit(-1);
	}
	if ((g_nofields = strlen(g_ofields)) > 2) {
		errorhandler("too many fields in OFIELDS");
		exit(-1);
	}
	if (g_nwdow <= 0)
		g_nwdow = g_n;
	if (g_nwdow >  g_n) {
		errorhandler("L > N not allowed");
		exit(-1);
	}
	if (g_m == 0)
		g_m = g_n;
	if (g_m > g_nwdow) {
		errorhandler("M > L not allowed");
		exit(-1);
	}
	if (g_ifiend == -2)
		g_ifiend = g_n/2;

	g_ifibeg = ((g_ifibeg % g_n) + g_n) % g_n;
	g_ifiend = ((g_ifiend % g_n) + g_n) % g_n;
	g_ifihop = ((g_ifihop % g_n) + g_n) % g_n;
	if (g_ifihop == 0) {
		errorhandler("illegal IHOPIDX");
		exit(-1);
	}
	/* 
	 * Tricky: trim end to be ((k-1)*hop+beg)%N == end with 0<k<=N.
	 * This is important for loop termination-condition somewhere.
	 * d is the number of indicies to be input if hop=1.
	 * k is the number of indicies to be input for given hop.
	 * hop, end, beg must be of non-negative modulo N, also hop > 0;
	 */
	d = ((g_ifiend - g_ifibeg + g_n) % g_n) + 1;
        k = (d + (g_ifihop - 1)) / g_ifihop;
	g_ifiend = ((k-1) * g_ifihop + g_ifibeg);
	g_ifiend = ((g_ifiend % g_n) + g_n) % g_n;

	nin = 2*k;			/* nin input numbers for block	*/

	fixflag = 0;			/* handle FIXANALYTIC		*/
	switch (g_fixa[0] | 0x20) {
	case 'a':
		if (g_ifibeg <= g_n/2)
		  if (g_ifibeg + (k-1)*g_ifihop <= g_n/2) 
		    fixflag = 1;
		if (g_ifibeg >= g_n/2)
		  if (g_ifibeg + (k-1)*g_ifihop <= g_n) 
		    fixflag = 1;
		break;
	case 'y':
		fixflag = 1; break;
	default:
		break;
	}

	/*
	 * Heuristics: scale = SCALE * L * (M/L) * L_analysis
	 * 2nd term accounts for scaling due to synthesis window
	 * 3rd term accounts for overlap effect
	 * 4th term accounts for scaling dur to analysis window
	 * (assume L = L_analysis)
	 */
	g_scale = g_scale_usr * g_m * g_nwdow;
	if (fixflag)			/* FIXANALYTIC: mply. all by 2	*/
		g_scale *= 2;		/* div. f-idx 0,N/2 by 2 later	*/

	strcpy(g_wtype, convspchar(g_wtype));
	strcpy(g_ofsep, convspchar(g_ofsep));
	strcpy(g_ofmt, convspchar(g_ofmt));
					/* allocate fftsize for complex */
	if ((fftp = fft_init(ldn))				== NULL ||
	    (wdow = window_init(fftsize*2))			== NULL ||
	    (in  = (double*)(malloc(sizeof(double)*fftsize*4)))	== NULL ||
	    (out = (double*)(malloc(sizeof(double)*fftsize*g_nofields)))	
								== NULL) {
		errorhandler("not enough memory");
		exit(-1);
	}
					/* open input file		*/
	istream = stdin;
	if (g_ifile[0] != '-' || g_ifile[1] != 0) {
		if ((istream = fopen(g_ifile, READ)) == NULL) {
			errmsg = catmsg(g_ifile, "cannot open");
			errorhandler(errmsg);
			exit(-1);
		}
	}
					/* open window file		*/
	wstream = NULL;
	if (g_wfile[0] != '-' || g_wfile[1] != 0) {
		if ((wstream = fopen(g_wfile, READ)) == NULL) {
			errmsg = catmsg(g_wfile, "cannot open");
			errorhandler(errmsg);
			exit(-1);
		}
	}
					/* open output ftt/ftp-file	*/
	splstream = NULL;
	if (g_splfile[0] != '-' || g_splfile[1] != 0) {
		if (istream != stdin) {
			errorhandler("SPLFILE/SPPFILE excludes use of IFILE");
			exit(-1);
		}
		istream = NULL;		/* mark IFILE disfunctional	*/
		if ((splstream = fopen(g_splfile, READ)) == NULL) {
			errmsg = catmsg(g_splfile, "cannot open");
			errorhandler(errmsg);
			exit(-1);
		}
	}
	sppstream = NULL;
	if (g_sppfile[0] != '-' || g_sppfile[1] != 0) {
		if (splstream == NULL) {
			errorhandler("SPPFILE demands SPLFILE");
			exit(-1);
		}
		if ((sppstream = fopen(g_sppfile, READ)) == NULL) {
			errmsg = catmsg(g_sppfile, "cannot open");
			errorhandler(errmsg);
			exit(-1);
		}
	}
					/* open output file/pipe	*/
	if (g_ofile[0] == 0) {
		errorhandler("OFILE must be specified");
		exit(-1);
	}
	ostream = stdout;
	if (g_ofile[0] != '-' || g_ofile[1] != 0) {
	    if(g_ofile[0] == '|') {
		if ((ostream = popen(g_ofile+1, WRITE)) == NULL) {
			errmsg = catmsg(g_ofile+1, "cannot execute");
			errorhandler(errmsg);
			exit(-1);
		}
	    } else {
		if ((ostream = fopen(g_ofile, WRITE)) == NULL) {
			errmsg = catmsg(g_ofile, "cannot open");
			errorhandler(errmsg);
			exit(-1);
		}
	    }
	}
					/* do the actual processing	*/
	do_calc(in, nin, fixflag, istream, wstream, wdow, fftp, out,
		ostream, splstream, sppstream);

					/* clean-up ...			*/
	if (splstream != NULL)
		fclose(splstream);
	if (sppstream != NULL)
		fclose(sppstream);
	if (istream != NULL)
		fclose(istream);
	if (wstream != NULL)
		fclose(wstream);
	if (ostream != NULL) {
	    if (g_ofile[0] == '|')
	        pclose(ostream);
	    else
	        fclose(ostream);
	}

	free(out);
	free(in);
	window_discard(wdow);
	fft_discard(fftp);
	exit(0);
}
/*******EOT**************************************************************/
