/*
 *   COPYRIGHT NOTICE: This code was written by Markus Mummert. All
 *   rights reserved. FOR LICENCE TERMS, SEE ACCOMPANYING FILE "LICENCE".
 */
/*************************************************************************
 *
 *   Name    : winhelp.c
 *   Version : 1999/08/25 1.3
 *   Comment : help-function on windows of module WINDOW
 *   Author  : Markus Mummert
 *   History :
 *   1999/08/22 1.1
 *	Taken out from DFT.C 1999/08/21 0.21
 *   1999/08/23 1.2
 *	Notes: separate multiple windows by `*' instead ` ';
 *	List: alias list width from 80 to 79
 *   1999/08/25 1.3
 *	List: output to stream instead of stderr
 *
 */

#include <stdio.h>
#include "portab.h"
#include "key.h"
#include "window.h"
#include "winhelp.h"

/*----------------------------------------------------------------------*/
/*  Show list								*/
/*----------------------------------------------------------------------*/
static show_list(stream, list, width)
FILE *stream;
char *list;
int width;
{
	int i, k, l, listitems, r, c, cols, rows, maxlen;
	char *p, w[MAXLINE+2], sf[10], pf[10];

	if (width > MAXLINE) {
		fprintf(stream, "*** show_list(): width too large ***\n");
		return;
	}
	sprintf (sf, "%%%ds%%n", MAXLINE+1);
	p = list; i = 0; listitems = 0; maxlen = 0;
	while (sscanf(p += i, sf, w, &i) > 0) {
	    l = strlen(w);
	    if (l > MAXLINE) {
		fprintf(stream, "*** show_list(): item too long ***\n");
		return;
	    }
	    if (l > maxlen)
		maxlen = l;
	    listitems++;
	}
	if (!listitems) {
		fprintf(stream, "*** show_list(): empty list ***\n");
		return;
	}
	cols = width/(maxlen+2);
	if (!cols) {
		fprintf(stream, "*** show_list(): width too small ***\n");
		return;
	}
	sprintf(pf, "%%-%ds", maxlen+2);
	rows = (listitems+(cols-1))/cols;
	for (r = 0; r < rows; r++) {
	    for (c = 0; c < cols; c++) {
		p = list; i = 0;
	        for (k = c*rows+r; k > 0; k--)
		    if (sscanf(p += i, sf, w, &i) <= 0)
			i = 0;
		if (sscanf(p += i, sf, w, &i) > 0)
		    fprintf(stream, pf, w);
	    }
	    fprintf(stream, "\n");
	}
}
/*----------------------------------------------------------------------*/
/*  Show window help							*/
/*----------------------------------------------------------------------*/
void help_on_windows(stream)
FILE *stream;
{
	fprintf(stream, "\
\n\
*** WINDOW help ***\n\
\n\
To see window-type names use: WINDOW list\n\
To read notes on windows use: WINDOW notes\n\
\n\
");
}
/*----------------------------------------------------------------------*/
/*  Show window list							*/
/*----------------------------------------------------------------------*/
void list_of_windows(stream)
FILE *stream;
{
	fprintf(stream, "\
\n\
*** WINDOW list ***\n\
\n\
");
	show_list(stream, WINDOW_LIST, 79);
	fprintf(stream, "\
\n\
");
	show_list(stream, WINDOW_ALIAS_LIST, 79);
	fprintf(stream, "\
\n\
");
}
/*----------------------------------------------------------------------*/
/*  Show window notes							*/
/*----------------------------------------------------------------------*/
void notes_on_windows(stream)
FILE *stream;
{
	fprintf(stream, "\
\n\
*** WINDOW notes ***\n\
\n\
Most types for WINDOW are described by Harris [Proc. IEEE 66(1), p51-83,\n\
1978] or by by Nuttall [IEEE ASSP-29(1), p84-91,1981]. Parameter `alpha'\n\
in Harris78 is identical to WALPHA, unless noted otherwise in the list below.\n\
The optimal window introduced by Adams [IEEE ASSP-39(8), p1753-1769, 1991]\n\
is not implemented but should be considered when discussing optimal window\n\
properties. For design rules using the versatile kaiser-bessel window see\n\
see Oppenheim and Schafer [Discrete Time-Signal Processing, Prentice-Hall,\n\
2nd edition, 1999].\n\
\n\
WSAMPLING determines if the endpoint of a sampled time-limited symmetric\n\
window function is part of the window sample vector (`symmetric') or not\n\
(`periodic'). chebyshev is `periodic' by definition, making it `symmetric'\n\
means having a zero as endpoint which is uncommon practice. For perfect\n\
reconstruction constraints in overlap-add-processing `periodic' likely is\n\
the right choice. In all other cases use `auto' as default. The difference\n\
between modes vanishes with increasing window size. For a discussion of\n\
sampling modes see Rohling and Schuerman [Signal Processing 5, 127-138,\n\
North-Holland, 1983].\n\
\n\
For WINDOW also multiple type names seperated by asterisks (`*') may be\n\
specified (e.g. WINDOW hanning*poisson), indicating that the types stated\n\
should be multiplied with each other. Note that the set of parameters\n\
(WALPHA, WF3DB, WFCUT, WSAMPLING, WSHIFT) will be the same for each type in\n\
the list. Thus one can not combine, e.g., a tukey of WALPHA=0.1 with a\n\
chebyshev of WALPHA=3.\n\
\n\
WRAISE may be used to trim sidelobes in some cases. The formula WRAISE+\n\
(1-WRAISE)*w[i]/wmax is applied after evaluating the window w[i] according\n\
to the specification of WINDOW; wmax is the maximum value of the w[i].\n\
\n\
<name>-X-Y-Z'			Refer to windows blackman-harris and nuttall\n\
				as X-term (mainlobe width is less or equal\n\
				to +/-X*L*f/FSAMP), main-to-sidelobe-level\n\
				Y dB, decay Z dB/octave. Corrections by\n\
				Nuttall81 on Harris78 are regarded.\n\
\n\
barcilon-temes			Based on frequency-domain specification of\n\
				a time-continuous window, thus some aliasing\n\
				involved. WALPHA deviates slightly from\n\
				alpha in Harris78.\n\
\n\
bX  (Bessel)			Derived from impulse-responses of time-con-\n\
p1  (single real pole)		tinuous causal lowpasses of order X. Time-\n\
Xp1 (X-fold real pole)		discrete realizations are obtained by\n\
pX  (Butterworth)		bilinear-transforming their pole plans using\n\
pgX (pseudo-gaussian-approx)	WF3DB/FSAMP as fixpoint. The L samples taken\n\
				from their realized impulse-responses start\n\
				at zero-time, but can be offset by WSHIFT\n\
				samples. The samples then are time-reversed\n\
			        to yield the effective window, e.g. WSHIFT=\n\
				+L/2 renders its right half to be zero.\n\
				Pole plans for pgX by Dishal [Electrical \n\
				Communication 36(1), p3-26, 1959].\n\
\n\
chebyshev			Eq.(45) in Harris78 corrected, compare\n\
				Diderich [Proc. IEEE 62, p1395-1396, 1974]\n\
\n\
exp   (single real pole)	Also derived from impulse-responses of time-\n\
tXexp (X+1-fold real pole)     	continuous causal lowpasses parametrized by\n\
				WF3DB. Now analytic representations of time-\n\
				continuous impulse-responses are directly\n\
				sampled. WSHIFT and time-reversal work as\n\
				above. Assuming WSHIFT=0 the effective window\n\
				for exp (n=1) and tXexp (n=X+1) is c*x^(n-1)*\n\
				exp(-x), x=pi*2*(L-1-i)*WF3DB/FSAMP/\n\
				sqrt(2^(1/n)-1), i=0...L-1.\n\
\n\
gaussian-f3db			c*exp(-x*x/log(2)/2), x=pi*(2*i-L)*WF3DB/\n\
				FSAMP, i=0...L-1. m=L-1 or m=L dependent on\n\
				WSAMPLING. Performs identical to gaussian\n\
				if WF3DB=0.265*WALPHA*FSAMP/L.\n\
\n\
kaiser-bessel			WALPHA as defined by Harris78. Most common is\n\
				beta = alpha' = pi*WALPHA.\n\
\n\
kaiser-bessel-fds		Based on frequency-domain specification of\n\
				time-continuous window, thus aliasing\n\
				involved. REPLACE WITH kaiser-bessel.\n\
\n\
modified-kaiser-bessel		As defined by Geckinly and Yavuz [IEEE\n\
				ASSP-26, p501-507, 1978], beta = pi*WALPHA.\n\
\n\
sinc				c*sin(x)/(x), x=pi*(2*i-m)*WFCUT/FSAMP, i=\n\
				0..L-1, m=L-1 or m=L dependent on WSAMPLING.\n\
\n\
tukey				Eq.(38) in Harris78 corrected, pictures ok.\n\
\n\
unraised-cosine-taper		As tukey (cosine-taper), but the slopes\n\
				adjacent to the constant center-part are\n\
				defined by quarter-periods of a cosine\n\
				rather than half-periods. WALPHA as tukey.\n\
\n\
van-der-maas			Based on frequency-domain specification of\n\
				time-continuous window, thus aliasing\n\
				involved. REPLACE WITH dolph-chebyshev.\n\
\n\
");
}

/*
 * EOT
 *
 ************************************************************************/
